@@ -22,10 +22,13 @@
 #include "core/or/trace_probes_cc.h"
 #include "feature/nodelist/networkstatus.h"
 #include "trunnel/flow_control_cells.h"
+#include "feature/control/control_events.h"
+#include "lib/math/stats.h"
 
 #include "core/or/connection_st.h"
 #include "core/or/cell_st.h"
 #include "app/config/config.h"
+#include "core/or/conflux_util.h"
 
 /** Cache consensus parameters */
 static uint32_t xoff_client;
@@ -35,6 +38,12 @@ static uint32_t xon_change_pct;
 static uint32_t xon_ewma_cnt;
 static uint32_t xon_rate_bytes;
 
+/** Metricsport stats */
+uint64_t cc_stats_flow_num_xoff_sent;
+uint64_t cc_stats_flow_num_xon_sent;
+double cc_stats_flow_xoff_outbuf_ma = 0;
+double cc_stats_flow_xon_outbuf_ma = 0;
+
 /* In normal operation, we can get a burst of up to 32 cells before returning
  * to libevent to flush the outbuf. This is a heuristic from hardcoded values
  * and strange logic in connection_bucket_get_share(). */
@@ -52,27 +61,6 @@ static uint32_t xon_rate_bytes;
 #define ONE_MEGABYTE (UINT64_C(1) << 20)
 #define TOTAL_XMIT_SCALE_AT (10 * ONE_MEGABYTE)
 
-/**
- * Return the congestion control object of the given edge connection.
- *
- * Returns NULL if the edge connection doesn't have a cpath_layer or not
- * attached to a circuit. But also if the cpath_layer or circuit doesn't have a
- * congestion control object.
- */
-static inline const congestion_control_t *
-edge_get_ccontrol(const edge_connection_t *edge)
-{
-  congestion_control_t *ccontrol = NULL;
-
-  if (edge->on_circuit && edge->on_circuit->ccontrol) {
-    ccontrol = edge->on_circuit->ccontrol;
-  } else if (edge->cpath_layer && edge->cpath_layer->ccontrol) {
-    ccontrol = edge->cpath_layer->ccontrol;
-  }
-
-  return ccontrol;
-}
-
 /**
  * Update global congestion control related consensus parameter values, every
  * consensus update.
@@ -89,7 +77,7 @@ flow_control_new_consensus_params(const networkstatus_t *ns)
   xoff_client = networkstatus_get_param(ns, "cc_xoff_client",
       CC_XOFF_CLIENT_DFLT,
       CC_XOFF_CLIENT_MIN,
-      CC_XOFF_CLIENT_MAX)*RELAY_PAYLOAD_SIZE;
+      CC_XOFF_CLIENT_MAX)*RELAY_PAYLOAD_SIZE_MIN;
 
 #define CC_XOFF_EXIT_DFLT 500
 #define CC_XOFF_EXIT_MIN 1
@@ -97,7 +85,7 @@ flow_control_new_consensus_params(const networkstatus_t *ns)
   xoff_exit = networkstatus_get_param(ns, "cc_xoff_exit",
       CC_XOFF_EXIT_DFLT,
       CC_XOFF_EXIT_MIN,
-      CC_XOFF_EXIT_MAX)*RELAY_PAYLOAD_SIZE;
+      CC_XOFF_EXIT_MAX)*RELAY_PAYLOAD_SIZE_MIN;
 
 #define CC_XON_CHANGE_PCT_DFLT 25
 #define CC_XON_CHANGE_PCT_MIN 1
@@ -113,7 +101,7 @@ flow_control_new_consensus_params(const networkstatus_t *ns)
   xon_rate_bytes = networkstatus_get_param(ns, "cc_xon_rate",
       CC_XON_RATE_BYTES_DFLT,
       CC_XON_RATE_BYTES_MIN,
-      CC_XON_RATE_BYTES_MAX)*RELAY_PAYLOAD_SIZE;
+      CC_XON_RATE_BYTES_MAX)*RELAY_PAYLOAD_SIZE_MAX;
 
 #define CC_XON_EWMA_CNT_DFLT (2)
 #define CC_XON_EWMA_CNT_MIN (2)
@@ -147,6 +135,14 @@ circuit_send_stream_xoff(edge_connection_t *stream)
   if (connection_edge_send_command(stream, RELAY_COMMAND_XOFF,
                                (char*)payload, (size_t)xoff_size) == 0) {
     stream->xoff_sent = true;
+    cc_stats_flow_num_xoff_sent++;
+
+    /* If this is an entry conn, notify control port */
+    if (TO_CONN(stream)->type == CONN_TYPE_AP) {
+      control_event_stream_status(TO_ENTRY_CONN(TO_CONN(stream)),
+                                  STREAM_EVENT_XOFF_SENT,
+                                  0);
+    }
   }
 }
 
@@ -213,6 +209,15 @@ circuit_send_stream_xon(edge_connection_t *stream)
                                    (size_t)xon_size) == 0) {
     /* Revert the xoff sent status, so we can send another one if need be */
     stream->xoff_sent = false;
+
+    cc_stats_flow_num_xon_sent++;
+
+    /* If it's an entry conn, notify control port */
+    if (TO_CONN(stream)->type == CONN_TYPE_AP) {
+      control_event_stream_status(TO_ENTRY_CONN(TO_CONN(stream)),
+                                  STREAM_EVENT_XON_SENT,
+                                  0);
+    }
   }
 }
 
@@ -220,17 +225,15 @@ circuit_send_stream_xon(edge_connection_t *stream)
  * Process a stream XOFF, parsing it, and then stopping reading on
  * the edge connection.
  *
- * Record that we have recieved an xoff, so we know not to resume
+ * Record that we have received an xoff, so we know not to resume
  * reading on this edge conn until we get an XON.
  *
  * Returns false if the XOFF did not validate; true if it does.
  */
 bool
 circuit_process_stream_xoff(edge_connection_t *conn,
-                            const crypt_path_t *layer_hint,
-                            const cell_t *cell)
+                            const crypt_path_t *layer_hint)
 {
-  (void)cell;
   bool retval = true;
 
   if (BUG(!conn)) {
@@ -240,13 +243,13 @@ circuit_process_stream_xoff(edge_connection_t *conn,
   }
 
   /* Make sure this XOFF came from the right hop */
-  if (layer_hint && layer_hint != conn->cpath_layer) {
+  if (!edge_uses_cpath(conn, layer_hint)) {
     log_fn(LOG_PROTOCOL_WARN, LD_EDGE,
             "Got XOFF from wrong hop.");
     return false;
   }
 
-  if (edge_get_ccontrol(conn) == NULL) {
+  if (!edge_uses_flow_control(conn)) {
     log_fn(LOG_PROTOCOL_WARN, LD_EDGE,
            "Got XOFF for non-congestion control circuit");
     return false;
@@ -299,6 +302,13 @@ circuit_process_stream_xoff(edge_connection_t *conn,
   connection_stop_reading(TO_CONN(conn));
   conn->xoff_received = true;
 
+  /* If this is an entry conn, notify control port */
+  if (TO_CONN(conn)->type == CONN_TYPE_AP) {
+    control_event_stream_status(TO_ENTRY_CONN(TO_CONN(conn)),
+                                STREAM_EVENT_XOFF_RECV,
+                                0);
+  }
+
   return retval;
 }
 
@@ -315,7 +325,7 @@ circuit_process_stream_xoff(edge_connection_t *conn,
 bool
 circuit_process_stream_xon(edge_connection_t *conn,
                            const crypt_path_t *layer_hint,
-                           const cell_t *cell)
+                           const relay_msg_t *msg)
 {
   xon_cell_t *xon;
   bool retval = true;
@@ -327,20 +337,19 @@ circuit_process_stream_xon(edge_connection_t *conn,
   }
 
   /* Make sure this XON came from the right hop */
-  if (layer_hint && layer_hint != conn->cpath_layer) {
+  if (!edge_uses_cpath(conn, layer_hint)) {
     log_fn(LOG_PROTOCOL_WARN, LD_EDGE,
            "Got XON from wrong hop.");
     return false;
   }
 
-  if (edge_get_ccontrol(conn) == NULL) {
+  if (!edge_uses_flow_control(conn)) {
     log_fn(LOG_PROTOCOL_WARN, LD_EDGE,
            "Got XON for non-congestion control circuit");
     return false;
   }
 
-  if (xon_cell_parse(&xon, cell->payload+RELAY_HEADER_SIZE,
-                     CELL_PAYLOAD_SIZE-RELAY_HEADER_SIZE) < 0) {
+  if (xon_cell_parse(&xon, msg->body, msg->length) < 0) {
     log_fn(LOG_PROTOCOL_WARN, LD_EDGE,
           "Received malformed XON cell.");
     return false;
@@ -403,6 +412,13 @@ circuit_process_stream_xon(edge_connection_t *conn,
     connection_start_reading(TO_CONN(conn));
   }
 
+  /* If this is an entry conn, notify control port */
+  if (TO_CONN(conn)->type == CONN_TYPE_AP) {
+    control_event_stream_status(TO_ENTRY_CONN(TO_CONN(conn)),
+                                STREAM_EVENT_XON_RECV,
+                                0);
+  }
+
   xon_cell_free(xon);
 
   return retval;
@@ -425,7 +441,7 @@ flow_control_decide_xoff(edge_connection_t *stream)
   size_t total_buffered = connection_get_outbuf_len(TO_CONN(stream));
   uint32_t buffer_limit_xoff = 0;
 
-  if (BUG(edge_get_ccontrol(stream) == NULL)) {
+  if (BUG(!edge_uses_flow_control(stream))) {
     log_err(LD_BUG, "Flow control called for non-congestion control circuit");
     return -1;
   }
@@ -444,6 +460,10 @@ flow_control_decide_xoff(edge_connection_t *stream)
                  total_buffered, buffer_limit_xoff);
       tor_trace(TR_SUBSYS(cc), TR_EV(flow_decide_xoff_sending), stream);
 
+      cc_stats_flow_xoff_outbuf_ma =
+        stats_update_running_avg(cc_stats_flow_xoff_outbuf_ma,
+                                 total_buffered);
+
       circuit_send_stream_xoff(stream);
 
       /* Clear the drain rate. It is considered wrong if we
@@ -457,7 +477,7 @@ flow_control_decide_xoff(edge_connection_t *stream)
    * do this because writes only happen when the socket unblocks, so
    * may not otherwise notice accumulation of data in the outbuf for
    * advisory XONs. */
-   if (total_buffered > MAX_EXPECTED_CELL_BURST*RELAY_PAYLOAD_SIZE) {
+   if (total_buffered > MAX_EXPECTED_CELL_BURST*RELAY_PAYLOAD_SIZE_MIN) {
      flow_control_decide_xon(stream, 0);
    }
 
@@ -598,6 +618,11 @@ flow_control_decide_xon(edge_connection_t *stream, size_t n_written)
                  stream->ewma_drain_rate,
                  total_buffered);
       tor_trace(TR_SUBSYS(cc), TR_EV(flow_decide_xon_rate_change), stream);
+
+      cc_stats_flow_xon_outbuf_ma =
+        stats_update_running_avg(cc_stats_flow_xon_outbuf_ma,
+                                 total_buffered);
+
       circuit_send_stream_xon(stream);
     }
   } else if (total_buffered == 0) {
@@ -669,21 +694,6 @@ edge_uses_flow_control(const edge_connection_t *stream)
   return ret;
 }
 
-/**
- * Returns the max RTT for the circuit that carries this stream,
- * as observed by congestion control.
- */
-uint64_t
-edge_get_max_rtt(const edge_connection_t *stream)
-{
-  if (stream->on_circuit && stream->on_circuit->ccontrol)
-    return stream->on_circuit->ccontrol->max_rtt_usec;
-  else if (stream->cpath_layer && stream->cpath_layer->ccontrol)
-    return stream->cpath_layer->ccontrol->max_rtt_usec;
-
-  return 0;
-}
-
 /** Returns true if a connection is an edge conn that uses flow control */
 bool
 conn_uses_flow_control(connection_t *conn)
@@ -700,4 +710,3 @@ conn_uses_flow_control(connection_t *conn)
 
   return ret;
 }
-
